' Title: Seven Open
' Author: Moochos
' Last modified: 2011-10-26

' On a release version:
'  - Update the variable "TApp.About_Version" (the version number)
'  - Update the variable "TApp.About_Copyright" (the copyright notation)
'  - Set the variable "TApp.Debugging" to False

' Restrictions:
'  - Compares paths in a manner of Windows NTs rather than of Windows 95s - 
'    full-width alphabets are case-insensitive
'  - Supports archive files as of 7-Zip File Manager 9.20 and 9.22 beta

Option Explicit
Const Debugging = False
Const About_Title = "Seven Open"
Const About_Version = "2.0.0"
Const About_Copyright = "Copyright (C) 2010-2011 Moochos. All rights reserved,"
Const Error_None = 0
Const Error_Error = &H80040001
Const Error_Abort = &H80040002
Dim FileSystem
Set FileSystem = CreateObject("Scripting.FileSystemObject")
Dim Shell
Set Shell = CreateObject("WScript.Shell")
Dim TargetPath
TargetPath = ""
Dim HelpPath
HelpPath = FileSystem.GetAbsolutePathName(FileSystem.BuildPath(WScript.ScriptFullName, "..\Help\index.html"))
Dim SettingsPath
SettingsPath = FileSystem.GetAbsolutePathName(FileSystem.BuildPath(WScript.ScriptFullName, "..\Settings.dat"))
Dim ArchiveExtensions
ArchiveExtensions = Split("001;7z;arj;bz2;bzip2;cab;cpio;deb;dmg;fat;gz;gzip;hfs;iso;lha;lzh;lzma;ntfs;rar;rpm;squashfs;swm;tar;taz;tbz;tbz2;tgz;tpz;txz;vhd;wim;xar;xz;z;zip", ";")
Dim ShortcutExtensions
ShortcutExtensions = Split("lnk", ";")
WScript.Quit Main

Function Main
	On Error Resume Next
	If Debugging Then
		On Error GoTo 0
	End If
	Start
	Select Case Err.Number
	Case Error_None
		' Do nothing
	Case Error_Abort
		' Do nothing
	Case Else
		MsgBox Err.Description, vbOKOnly Or vbCritical, About_Title
	End Select
	Main = Err.Number
End Function

Sub Start
	LoadSettings
	DetermineTask.Perform
End Sub

Function DetermineTask
	If WScript.Arguments.Unnamed.Count = 0 Then
		Do
			Select Case Trim(InputBox("Specify a command: 1 - offline help, 2 - about, 3 - edit settings, 4 - reset settings.", About_Title, "1"))
			Case "1"
				Set DetermineTask = New THelpTask
			Case "2"
				Set DetermineTask = New TAboutTask
			Case "3"
				Set DetermineTask = New TEditSettingsTask
			Case "4"
				Set DetermineTask = New TResetSettingsTask
			Case Empty
				Abort
			End Select
		Loop While IsEmpty(DetermineTask)
	Else
		Set DetermineTask = New TOpenTask
	End If
End Function

Sub LoadSettings
	If Not FileSystem.FileExists(SettingsPath) Then
		Exit Sub
	End If
	Dim Settings
	Set Settings = New TSettings
	Settings.Load SettingsPath
	SetSettings(Settings)
End Sub

Sub SaveSettings
	Dim Settings
	Set Settings = New TSettings
	GetSettings(Settings)
	Settings.Save SettingsPath
End Sub

Sub GetSettings(Settings)
	Settings.Clear
	Settings.Write "TargetPath", TargetPath
	Settings.Write "ArchiveExtensions", Join(ArchiveExtensions, ";")
End Sub

Sub SetSettings(Settings)
	TargetPath = Settings.Read("TargetPath", TargetPath)
	ArchiveExtensions = Split(Settings.Read("ArchiveExtensions", Join(ArchiveExtensions, ";")), ";")
End Sub

Sub Error(Description)
	Err.Clear
	Err.Raise Error_Error, About_Title, Description
End Sub

Sub Abort
	Err.Clear
	Err.Raise Error_Abort, About_Title, "The operation was aborted."
End Sub

Class THelpTask
	Public Sub Perform
		Shell.Run HelpPath
	End Sub
End Class

Class TAboutTask
	Public Sub Perform
		Dim Lines(2)
		Lines(0) = About_Title & " " & About_Version
		Lines(1) = ""
		Lines(2) = About_Copyright
		MsgBox Join(Lines, vbNewLine), vbOKOnly, About_Title
	End Sub
End Class

Class TEditSettingsTask
	Public Sub Perform
		Dim Settings
		Set Settings = New TSettings
		GetSettings(Settings)

		Dim Key
		Dim Input

		Key = "TargetPath"
		Input = InputBox("Specify the path of 7-Zip File Manager.", About_Title, Settings.Read(Key, ""))
		If IsEmpty(Input) Then
			Abort
		End If
		Settings.Write Key, Input

		Key = "ArchiveExtensions"
		Input = InputBox("Specify file name extensions for archives (separate with semicolons).", About_Title, Settings.Read(Key, ""))
		If IsEmpty(Input) Then
			Abort
		End If
		Settings.Write Key, Input

		Input = Empty
		Key = Empty

		If MsgBox("The program updates the settings.", vbOKCancel Or vbExclamation, About_Title) <> vbOK Then
			Abort
		End If
		SetSettings Settings
		Set Settings = Nothing
		SaveSettings
		MsgBox "The program updated the settings.", vbOKOnly Or vbInformation, About_Title
	End Sub
End Class

Class TResetSettingsTask
	Public Sub Perform
		If MsgBox("The program resets the settings.", vbOKCancel Or vbExclamation, About_Title) <> vbOK Then
			Abort
		End If
		FileSystem.DeleteFile SettingsPath
		MsgBox "The program reset the settings.", vbOKOnly Or vbInformation, About_Title
	End Sub
End Class

Class TOpenTask
	Public Sub Perform
		EnsureTargetExists
		Dim Paths
		Set Paths = WScript.Arguments.Unnamed
		Dim Path
		For Each Path In Paths
			Process Path
		Next
	End Sub

	Private Sub EnsureTargetExists
		Do Until FileSystem.FileExists(TargetPath)
			MsgBox "The program cannot find the 7-Zip File Manager." & vbNewLine & vbNewLine & "Edit the settings and proceed.", vbOKOnly Or vbExclamation, About_Title
			Call (New TEditSettingsTask).Perform
		Loop
	End Sub

	Private Sub Process(Path)
		Dim SourcePath
		SourcePath = DetermineSourcePath(Path)
		Dim Parameters(2)
		Parameters(0) = """" & TargetPath & """"
		Parameters(1) = """" & SourcePath & """"
		If Shell.Exec(Join(Parameters, " ")).ProcessID = 0 Then
			Error "The program could not open the file """ & SourcePath & """ with 7-Zip File Manager."
		End If
	End Sub

	Private Function DetermineSourcePath(Path)
		Dim SourcePath
		If Len(FileSystem.GetDriveName(Path)) = 0 Then
			Error "The program cannot accept relative paths: """ & Path & """."
		End If
		SourcePath = FileSystem.GetAbsolutePathName(Path)
		If FileSystem.FileExists(SourcePath) Then
			If FileTypeMatches(SourcePath, ShortcutExtensions) Then
				SourcePath = DetermineSourcePath(Shell.CreateShortcut(SourcePath).TargetPath)
			ElseIf FileTypeMatches(SourcePath, ArchiveExtensions) Then
				' Do nothing
			Else
				SourcePath = FileSystem.GetAbsolutePathName(FileSystem.BuildPath(SourcePath, ".."))
			End If
		ElseIf FileSystem.FolderExists(SourcePath) Then
			' Do nothing
		Else
			Error "The program cannot find the file """ & SourcePath & """."
		End If
		DetermineSourcePath = SourcePath
	End Function

	Private Function FileTypeMatches(Path, Extensions)
		Dim Extension1
		Extension1 = FileSystem.GetExtensionName(FileSystem.GetAbsolutePathName(Path))
		Dim Extension2
		For Each Extension2 In Extensions
			If ComparePath(Extension1, Extension2) = 0 Then
				FileTypeMatches = True
				Exit Function
			End If
		Next
		FileTypeMatches = False
	End Function

	Private Function ComparePath(Path1, Path2)
		ComparePath = StrComp(Path1, Path2, vbTextCompare)
	End Function
End Class

' Designed as a portable class
Class TSettings
	Public Function Read(Key, DefaultValue)
		If Dictionary.Exists(Key) Then
			Read = Dictionary(Key)
		Else
			Read = DefaultValue
		End If
	End Function

	Public Sub Write(Key, Value)
		If VarType(Value) <> vbString Then
			Err.Raise vbObjectError + 1, "TSettings", "The program cannot write any non-string values to the settings."
		End If
		If Dictionary.Exists(Key) Then
			Dictionary(Key) = Value
		Else
			Dictionary.Add Key, Value
		End If
	End Sub

	Public Sub Clear
		Dictionary.RemoveAll
	End Sub

	Public Sub Load(Path)
		Dim NewDictionary
		Set NewDictionary = CreateObject("Scripting.Dictionary")
		Dim File
		Set File = FileSystem.OpenTextFile(Path, 1, False)
		Dim Parser
		Set Parser = New RegExp
		Parser.Pattern = "([^=]*)=([\s\S]*)"
		Do Until File.AtEndOfStream
			Dim Entry
			Set Entry = Parser.Execute(File.ReadLine)(0).SubMatches
			NewDictionary.Add Entry(0), Entry(1)
			Set Entry = Nothing
		Loop
		Set Parser = Nothing
		Set File = Nothing
		Set Dictionary = NewDictionary
	End Sub

	Public Sub Save(Path)
		Dim File
		Set File = FileSystem.OpenTextFile(Path, 2, True)
		Dim Keys
		Keys = Dictionary.Keys
		Dim Key
		For Each Key In Keys
			File.WriteLine Key & "=" & Dictionary(Key)
		Next
	End Sub

	Private Sub Class_Initialize
		Set Dictionary = CreateObject("Scripting.Dictionary")
		Dictionary.CompareMode = vbTextCompare
		Set FileSystem = CreateObject("Scripting.FileSystemObject")
	End Sub

	Private Dictionary
	Private FileSystem
End Class
